;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; simpleboot.asm   v0.00.01
;; NASM Assembly
;;
;; (C) 1998-2002, Particle
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; when the machine starts up, we have:
;; F0000h - FFFFFh ROM BIOS control basic hardware, etc.
;; D0000h - EFFFFh free for additional BIOS extensions (usually)
;; C0000h - CFFFFh for BIOS extensions (disk controllers, etc.)
;; A0000h - BFFFFh for video memory
;; 00600h - 9FFFFh for user code
;; 00400h - 005FFh for bios data area
;; 00000h - 003FFh for interrupt table
;;
;; usable: A0000h - 00600h = 9FA00h = 653824 bytes
;; addressable: FFFFFh = 1,048,575 bytes
;; max segment length: FFFFh = 65535 bytes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

start:

;; when the machine starts up, BIOS puts first sector
;; (512 <200h> bytes) of  booting media into location 07C00h.

        mov     ax,0x07C0       ; where bios loaded out program
        mov     ds,ax
        mov     ss,ax           ; use the same seg for stack
        mov     sp,0x400      ; small stack

        mov     bx,helloworld
        call    printstr        ; print message

.loop:  jmp     .loop           ; halt

; our data
helloworld      db      13,10,'Hello World!',0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; print a character (OS independent)...
;; input: AX = char
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printchar:
        push    ax
        push    bx
        mov     ah,0x0E         ; print a char & advance position
        mov     bx,0x0007       ; set page = 0, and attribute = 7
        int     0x10            ; call BIOS
        pop     bx
        pop     ax
        ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; print string (null terminated)
;; input: BX = pointer to beg of string
;; (note: this method is kind of weird, it's kind of recursive,
;;      yet, it's not recursive... hmm...)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printstr:
        mov     al,[bx]                 ; get string char
        cmp     al,0                    ; is it 0?
        je      printstr.done        ; if yes, we're done
        call    printchar               ; else, print that char
        inc     bx                      ; increment pointer
        jmp     printstr             ; repeat
.done   ret                             ; return


